@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileCopy.cc,v 1.161 2011/04/11 09:23:32 lav Exp $ */
+/* $Id: FileCopy.cc,v 1.162 2011/04/22 15:43:11 lav Exp $ */
 
 /* FileCopyPeer behaviour:
     1) when suspended, does nothing
@@ -644,7 +643,8 @@ void FileCopy::LogTransfer()
    long long range_limit=GetRangeLimit();
    if(range_limit==FILE_END)
       range_limit=get->GetPos();
-   transfer_log->Format(0,"%s -> %s %lld-%lld %s\n",src,dst,
+   transfer_log->Format(0,"%s -> %s %lld-%lld %s\n",
+      url::remove_password(src),url::remove_password(dst),
       (long long)GetRangeStart(),range_limit,
       Speedometer::GetStr(GetBytesCount()/GetTimeSpent()).get());
 }
@@ -1561,6 +1561,10 @@ int FileCopyPeerFDStream::Get_LL(int len)
       {
 	 SetDate(st.st_mtime);
 	 SetSize(st.st_size);
+#ifndef NATIVE_CRLF
+	 if(ascii)
+	    SetSize(NO_SIZE);
+#endif
       }
    }
 
