@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1998-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: FileSet.cc,v 1.74 2011/03/25 14:35:41 lav Exp $ */
+/* $Id: FileSet.cc,v 1.75 2011/04/29 04:58:27 lav Exp $ */
 
 #include <config.h>
 #include "FileSet.h"
@@ -713,7 +712,7 @@ check_again:
 
    if(t==SYMLINK)
    {
-      char *buf=(char*)alloca(st.st_size+1);
+      char *buf=string_alloca(st.st_size+1);
       int res=readlink(name,buf,st.st_size);
       if(res!=-1)
       {
@@ -936,7 +935,7 @@ void FileInfo::MakeLongName()
    int w=20-strlen(usergroup);
    if(w<1)
       w=1;
-   char size_str[20];
+   char size_str[21];
    if(defined&SIZE)
       snprintf(size_str,sizeof(size_str),"%*lld",w,(long long)size);
    else
