@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 2000-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 #include <config.h>
@@ -219,7 +218,7 @@ int Fish::Do()
 	 if(xstrcmp(path_queue.LastString(),cwd))
 	 {
 	    Send("#CWD %s\n"
-		 "cd %s; echo '### 000'\n",cwd.path.get(),shell_encode(cwd));
+		 "cd %s; echo '### 000'\n",cwd.path.get(),shell_encode(cwd).get());
 	    PushExpect(EXPECT_CWD);
 	    PushDirectory(cwd);
 	 }
@@ -565,7 +564,7 @@ int Fish::HandleReplies()
 	 const char *eol=strchr(err,'\n');
 	 if(eol) {
 	    xstring &e=xstring::get_tmp(err,eol-err);
-	    LogError(0,e);
+	    LogError(0,"%s",e.get());
 	    SetError(NO_FILE,e);
 	    if(pty_recv_buf)
 	       pty_recv_buf->Skip(eol-err+1);
@@ -640,8 +639,6 @@ int Fish::HandleReplies()
    expect_t e=RespQueue.next();
 
    bool keep_message=false;
-   xstring p;
-
    switch(e)
    {
    case EXPECT_FISH:
@@ -658,7 +655,8 @@ int Fish::HandleReplies()
 	 set_home(home_auto);
       cache->SetDirectory(this, home, true);
       break;
-   case EXPECT_CWD:
+   case EXPECT_CWD: {
+      xstring p;
       PopDirectory(&p);
       if(message==0)
       {
@@ -673,6 +671,7 @@ int Fish::HandleReplies()
       else
 	 SetError(NO_FILE,message);
       break;
+   }
    case EXPECT_RETR_INFO:
       if(message && is_ascii_digit(message[0]) && !strchr(message,':'))
       {
@@ -684,13 +683,13 @@ int Fish::HandleReplies()
 	       *opt_size=entity_size;
 	 }
       }
-      else if(message)
+      else if(message && message[0]!='#')
       {
 	 FileInfo *fi=FileInfo::parse_ls_line(message,"GMT");
 	 if(!fi)
 	 {
 	    SetError(NO_FILE,message);
-	    return MOVED;
+	    break;
 	 }
 	 if(fi->defined&fi->SIZE)
 	 {
