@@ -1,7 +1,7 @@
 /*
  * lftp - file transfer program
  *
- * Copyright (c) 1999-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: NetAccess.cc,v 1.111 2011/03/25 14:35:42 lav Exp $ */
+/* $Id: NetAccess.cc,v 1.112 2011/06/16 11:08:42 lav Exp $ */
 
 #include <config.h>
 
@@ -59,6 +58,7 @@ void NetAccess::Init()
    connection_takeover=false;
 
    Reconfig(0);
+   reconnect_interval_current=reconnect_interval;
 }
 
 NetAccess::NetAccess()
