@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: ResMgr.cc,v 1.81 2011/03/25 14:35:42 lav Exp $ */
+/* $Id: ResMgr.cc,v 1.82 2011/05/10 05:53:20 lav Exp $ */
 
 #include <config.h>
 
@@ -479,7 +478,7 @@ const char *ResMgr::UNumberValidate(xstring_c *value)
 
    return 0;
 }
-unsigned long long ResValue::to_unumber(unsigned long long max)
+unsigned long long ResValue::to_unumber(unsigned long long max) const
 {
    if (is_nil())
       return 0;
@@ -491,7 +490,7 @@ unsigned long long ResValue::to_unumber(unsigned long long max)
       return max;
    return vm;
 }
-long long ResValue::to_number(long long min,long long max)
+long long ResValue::to_number(long long min,long long max) const
 {
    if (is_nil())
       return 0;
@@ -507,22 +506,28 @@ long long ResValue::to_number(long long min,long long max)
       return min;
    return vm;
 }
-ResValue::operator int()
+ResValue::operator int() const
 {
    return to_number(INT_MIN,INT_MAX);
 }
-ResValue::operator long()
+ResValue::operator long() const
 {
    return to_number(LONG_MIN,LONG_MAX);
 }
-ResValue::operator unsigned()
+ResValue::operator unsigned() const
 {
    return to_unumber(UINT_MAX);
 }
-ResValue::operator unsigned long()
+ResValue::operator unsigned long() const
 {
    return to_unumber(ULONG_MAX);
 }
+bool ResValue::to_tri_bool(bool a) const
+{
+   if(*s=='a' || *s=='A')
+      return a;
+   return to_bool();
+}
 
 ResMgr::Resource::Resource(Resource *next,const ResType *type,const char *closure,const char *value)
    : type(type), value(value), closure(closure), next(next)
@@ -538,8 +543,16 @@ bool ResMgr::Resource::ClosureMatch(const char *cl_data)
       return true;
    if(!(closure && cl_data))
       return false;
-   return 0==fnmatch(closure,cl_data,FNM_PATHNAME)
-      || (closure[0]=='*' && closure[1]=='.' && !strcmp(closure+2,cl_data));
+   // a special case for domain name match (i.e. example.org matches *.example.org)
+   if(closure[0]=='*' && closure[1]=='.' && !strcmp(closure+2,cl_data))
+      return true;
+   if(0==fnmatch(closure,cl_data,FNM_PATHNAME))
+      return true;
+   // try to match basename; helps matching torrent metadata url to *.torrent
+   const char *bn=basename_ptr(cl_data);
+   if(bn!=cl_data && 0==fnmatch(closure,bn,FNM_PATHNAME))
+      return true;
+   return false;
 }
 
 const char *ResMgr::QueryNext(const char *name,const char **closure,Resource **ptr)
@@ -657,24 +670,14 @@ ResType::~ResType()
 
    {
       // remove all resources of this type
-      bool modified=false;
       ResMgr::Resource **scan=&ResMgr::chain;
       while(*scan)
       {
 	 if((*scan)->type==this)
-	 {
 	    delete replace_value(*scan,(*scan)->next);
-	    modified=true;
-	 }
 	 else
-	 {
 	    scan=&(*scan)->next;
-	 }
       }
-#if 0 // this makes trouble at exit.
-      if(modified)
-	 SMTask::ReconfigAll(this->name);
-#endif
    }
 }
 
@@ -814,8 +817,9 @@ const char *ResMgr::ERegExpValidate(xstring_c *s)
    int err=regcomp(&re,*s,REG_EXTENDED|REG_NOSUB);
    if(err)
    {
-      static char err_msg[128];
-      regerror(err,0,err_msg,sizeof(err_msg));
+      const int max_err_len=128;
+      char *err_msg=xstring::tmp_buf(max_err_len);
+      regerror(err,0,err_msg,max_err_len);
       return err_msg;
    }
    regfree(&re);
@@ -938,7 +942,7 @@ const char *ResMgr::UNumberPairValidate(xstring_c *value)
 
    return 0;
 }
-void ResValue::ToNumberPair(int &a,int &b)
+void ResValue::ToNumberPair(int &a,int &b) const
 {
    switch(sscanf(s,"%d%*c%d",&a,&b))
    {
@@ -986,3 +990,15 @@ bool ResClient::QueryBool(const char *name,const char *closure) const
 {
    return Query(name,closure).to_bool();
 }
+bool ResType::QueryTriBool(const char *closure,bool a) const
+{
+   return Query(closure).to_tri_bool(a);
+}
+bool ResMgr::QueryTriBool(const char *name,const char *closure,bool a)
+{
+   return Query(name,closure).to_tri_bool(a);
+}
+bool ResClient::QueryTriBool(const char *name,const char *closure,bool a) const
+{
+   return Query(name,closure).to_tri_bool(a);
+}
