@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 2009-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Torrent.cc,v 1.48 2011/03/10 07:00:04 lav Exp $ */
+/* $Id: Torrent.cc,v 1.54 2011/08/01 09:51:51 lav Exp $ */
 
 #include <config.h>
 #include <stdlib.h>
@@ -46,6 +45,7 @@ static ResType torrent_vars[] = {
    {"torrent:seed-max-time", "30d", ResMgr::TimeIntervalValidate},
    {"torrent:seed-min-peers", "3", ResMgr::UNumberValidate},
    {"torrent:ip", "", ResMgr::IPv4AddrValidate, ResMgr::NoClosure},
+   {"torrent:retracker", ""},
 #if INET6
    {"torrent:ipv6", "", ResMgr::IPv6AddrValidate, ResMgr::NoClosure},
 #endif
@@ -53,6 +53,54 @@ static ResType torrent_vars[] = {
 };
 static ResDecls torrent_vars_register(torrent_vars);
 
+#if INET6
+#ifdef HAVE_IFADDRS_H
+#include <ifaddrs.h>
+#endif
+static const char *FindGlobalIPv6Address()
+{
+#ifdef HAVE_IFADDRS_H
+   struct ifaddrs *ifaddrs=0;
+   getifaddrs(&ifaddrs);
+   for(struct ifaddrs *ifa=ifaddrs; ifa; ifa=ifa->ifa_next) {
+      if(ifa->ifa_addr && ifa->ifa_addr->sa_family==AF_INET6) {
+	 struct in6_addr *addr=&((struct sockaddr_in6*)ifa->ifa_addr)->sin6_addr;
+	 if(!IN6_IS_ADDR_UNSPECIFIED(addr) && !IN6_IS_ADDR_LOOPBACK(addr)
+	 && !IN6_IS_ADDR_LINKLOCAL(addr) && !IN6_IS_ADDR_SITELOCAL(addr)
+	 && !IN6_IS_ADDR_MULTICAST(addr)) {
+	    char *buf=xstring::tmp_buf(INET6_ADDRSTRLEN);
+	    inet_ntop(AF_INET6, addr, buf, INET6_ADDRSTRLEN);
+	    freeifaddrs(ifaddrs);
+	    return buf;
+	 }
+      }
+   }
+   freeifaddrs(ifaddrs);
+#endif
+   return 0;
+}
+#endif
+
+void Torrent::ClassInit()
+{
+   static bool inited;
+   if(inited)
+      return;
+   inited=true;
+
+#if INET6
+   const char *ipv6=ResMgr::Query("torrent:ipv6",0);
+   if(!*ipv6)
+   {
+      ipv6=FindGlobalIPv6Address();
+      if(ipv6) {
+	 ProtoLog::LogNote(9,"found IPv6 address: %s",ipv6);
+	 ResMgr::Set("torrent:ipv6",0,ipv6);
+      }
+   }
+#endif
+}
+
 xstring Torrent::my_peer_id;
 xstring Torrent::my_key;
 xmap<Torrent*> Torrent::torrents;
@@ -103,6 +151,7 @@ Torrent::Torrent(const char *mf,const char *c,const char *od)
    total_recv=0;
    total_left=0;
    complete_pieces=0;
+   connected_peers_count=0;
    active_peers_count=0;
    complete_peers_count=0;
    am_interested_peers_count=0;
@@ -123,8 +172,8 @@ Torrent::Torrent(const char *mf,const char *c,const char *od)
 #endif
    }
    if(!my_peer_id) {
-      my_peer_id.set("-lftp42-");
-      my_peer_id.appendf("%04x",(unsigned)getpid());
+      my_peer_id.set("-lftp43-");
+      my_peer_id.appendf("%04x",(unsigned)getpid() & 0xffff);
       my_peer_id.appendf("%08x",(unsigned)now.UnixTime());
       assert(my_peer_id.length()==PEER_ID_LEN);
    }
@@ -157,6 +206,9 @@ int Torrent::Done() const
 
 void TorrentTracker::Shutdown()
 {
+   if(Failed()) // don't stop a failed tracker
+      return;
+   // stop if have started or at least processing a start request
    if(started || tracker_reply)
       SendTrackerRequest("stopped");
 }
@@ -370,6 +422,7 @@ int TorrentTracker::HandleTrackerReply()
 
    if(reply->type!=BeNode::BE_DICT) {
       SetError("Reply: wrong reply type, must be DICT");
+      tracker_reply=0;
       return MOVED;
    }
 
@@ -379,6 +432,7 @@ int TorrentTracker::HandleTrackerReply()
 	 SetError(b_failure_reason->str);
       else
 	 SetError("Reply: wrong `failure reason' type, must be STR");
+      tracker_reply=0;
       return MOVED;
    }
 
@@ -560,6 +614,8 @@ int Torrent::Do()
          return MOVED;
       }
 
+      const char *retracker=ResMgr::Query("torrent:retracker",GetName());
+      int retracker_len=xstrlen(retracker);
       BeNode *announce_list=metainfo_tree->dict.lookup("announce-list");
       if(announce_list && announce_list->type==BeNode::BE_LIST) {
 	 for(int i=0; i<announce_list->list.length(); i++) {
@@ -571,6 +627,8 @@ int Torrent::Do()
 	       BeNode *announce=announce_list1->list[j];
 	       if(announce->type!=BeNode::BE_STR)
 		  continue;
+	       if(retracker_len && !strncmp(retracker,announce->str,retracker_len))
+		  retracker=0, retracker_len=0;
 	       if(!new_tracker)
 		  new_tracker=new TorrentTracker(this,announce->str);
 	       else
@@ -595,6 +653,14 @@ int Torrent::Do()
 	 }
       }
 
+      if(retracker_len) {
+	 SMTaskRef<TorrentTracker> new_tracker(new TorrentTracker(this,retracker));
+	 if(!new_tracker->Failed()) {
+	    new_tracker->tracker_no=trackers.count();
+            trackers.append(new_tracker.borrow());
+	 }
+      }
+
       info=Lookup(metainfo_tree,"info",BeNode::BE_DICT);
       if(!info)
          return MOVED;
@@ -672,6 +738,7 @@ int Torrent::Do()
       if(!force_valid) {
 	 validate_index=0;
 	 validating=true;
+	 recv_rate.Reset();
       } else {
 	 for(unsigned i=0; i<total_pieces; i++)
 	    my_bitfield->set_bit(i,1);
@@ -683,9 +750,13 @@ int Torrent::Do()
    }
    if(validating) {
       ValidatePiece(validate_index++);
-      if(validate_index<total_pieces)
+      if(validate_index<total_pieces) {
+	 recv_rate.Add(piece_length);
 	 return MOVED;
+      }
+      recv_rate.Add(last_piece_length);
       validating=false;
+      recv_rate.Reset();
       if(total_left==0) {
 	 complete=true;
 	 seed_timer.Reset();
@@ -700,9 +771,11 @@ int Torrent::Do()
       OptimisticUnchoke();
 
    // count peers
+   connected_peers_count=0;
    active_peers_count=0;
    complete_peers_count=0;
    for(int i=0; i<peers.count(); i++) {
+      connected_peers_count+=peers[i]->Connected();
       active_peers_count+=peers[i]->Active();
       complete_peers_count+=peers[i]->Complete();
    }
@@ -1005,21 +1078,25 @@ int FDCache::OpenFile(const char *file,int m,off_t size)
    } while(fd==-1 && (errno==EMFILE || errno==ENFILE) && CloseOne());
    FD new_entry = {fd,errno,now.UnixTime()};
    cache[ci].add(file,new_entry);
-#ifdef HAVE_POSIX_FALLOCATE
+   if(fd!=-1)
+      fcntl(fd,F_SETFD,FD_CLOEXEC);
    if(fd==-1 || size==0)
       return fd;
+#ifdef HAVE_POSIX_FALLOCATE
    if(ci==O_RDWR) {
       struct stat st;
       // check if it is newly created file, then allocate space
       if(fstat(fd,&st)!=-1 && st.st_size==0)
 	 posix_fallocate(fd,0,size);
    }
-   else if(ci==O_RDONLY) {
-      // validation mode
+#endif//HAVE_POSIX_FALLOCATE
+#ifdef HAVE_POSIX_FADVISE
+   if(ci==O_RDONLY) {
+      // validation mode (when validating, size>0)
       posix_fadvise(fd,0,size,POSIX_FADV_SEQUENTIAL);
       posix_fadvise(fd,0,size,POSIX_FADV_NOREUSE);
    }
-#endif//HAVE_POSIX_FALLOCATE
+#endif//HAVE_POSIX_FADVISE
    return fd;
 }
 
@@ -1337,10 +1414,11 @@ void Torrent::Reconfig(const char *name)
 const xstring& Torrent::Status()
 {
    if(metainfo_data)
-      return xstring::format("Getting meta-data: %s",metainfo_data->Status());
+      return xstring::format(_("Getting meta-data: %s"),metainfo_data->Status());
    if(validating) {
-      return xstring::format("Validation: %u/%u (%u%%)",validate_index,total_pieces,
-	 validate_index*100/total_pieces);
+      return xstring::format(_("Validation: %u/%u (%u%%) %s%s"),validate_index,total_pieces,
+	 validate_index*100/total_pieces,recv_rate.GetStrS(),
+	 recv_rate.GetETAStrFromSize((off_t)(total_pieces-validate_index-1)*piece_length+last_piece_length).get());
    }
    if(shutting_down) {
       if(trackers.count()==0)
@@ -1348,7 +1426,7 @@ const xstring& Torrent::Status()
       for(int i=0; i<trackers.count(); i++) {
 	 const char *status=trackers[i]->Status();
 	 if(status[0]) {
-	    xstring &s=xstring::get_tmp("Shutting down: ");
+	    xstring &s=xstring::get_tmp(_("Shutting down: "));
 	    if(trackers.count()>1)
 	       s.appendf("%d. ",i+1);
 	    s.append(status);
@@ -2005,9 +2083,9 @@ int TorrentPeer::Do()
       if(s!=UNPACK_SUCCESS) {
 	 if(s==UNPACK_PREMATURE_EOF) {
 	    if(recv_buf->Size()>0)
-	       LogError(2,"peer unexpectedly closed connection after %s",recv_buf->Dump());
+	       LogError(2,_("peer unexpectedly closed connection after %s"),recv_buf->Dump());
 	    else
-	       LogError(4,"peer closed connection");
+	       LogError(4,_("peer closed connection (before handshake)"));
 	 }
 	 Disconnect();
 	 return MOVED;
@@ -2080,7 +2158,7 @@ int TorrentPeer::Do()
    }
 
    if(recv_buf->Eof() && recv_buf->Size()==0) {
-      LogError(4,"peer closed connection");
+      LogError(4,_("peer closed connection"));
       Disconnect();
       return MOVED;
    }
@@ -2092,9 +2170,9 @@ int TorrentPeer::Do()
    if(st!=UNPACK_SUCCESS)
    {
       if(st==UNPACK_PREMATURE_EOF)
-	 LogError(2,"peer unexpectedly closed connection after %s",recv_buf->Dump());
+	 LogError(2,_("peer unexpectedly closed connection after %s"),recv_buf->Dump());
       else
-	 LogError(2,"invalid peer response format");
+	 LogError(2,_("invalid peer response format"));
       Disconnect();
       return MOVED;
    }
@@ -2218,11 +2296,11 @@ const char *TorrentPeer::GetName() const
 const char *TorrentPeer::Status()
 {
    if(sock==-1)
-      return "Not connected";
+      return _("Not connected");
    if(!connected)
-      return "Connecting...";
+      return _("Connecting...");
    if(!peer_id)
-      return "Handshaking...";
+      return _("Handshaking...");
    xstring &buf=xstring::format("dn:%llu %sup:%llu %s",
       peer_recv,peer_recv_rate.GetStrS(),peer_sent,peer_send_rate.GetStrS());
    if(peer_interested)
@@ -2457,7 +2535,7 @@ int TorrentListener::Do()
 	       TimeoutS(1);
 	       return m;
 	    }
-	    error=Error::Fatal("Cannot bind a socket for torrent:port-range");
+	    error=Error::Fatal(_("Cannot bind a socket for torrent:port-range"));
 	    return MOVED;
 	 }
 	 LogError(10,"bind(%s): %s",addr.to_string().get(),strerror(saved_errno));
@@ -2485,7 +2563,7 @@ int TorrentListener::Do()
       return m;
    }
    rate.Add(1);
-   LogNote(3,"Accepted connection from [%s]:%d",remote_addr.address(),remote_addr.port());
+   LogNote(3,_("Accepted connection from [%s]:%d"),remote_addr.address(),remote_addr.port());
    (void)new TorrentDispatcher(a,&remote_addr);
    m=MOVED;
 
@@ -2519,7 +2597,7 @@ int TorrentDispatcher::Do()
 {
    if(timeout_timer.Stopped())
    {
-      LogError(1,"peer handshake timeout");
+      LogError(1,_("peer handshake timeout"));
       deleting=true;
       return MOVED;
    }
@@ -2531,9 +2609,9 @@ int TorrentDispatcher::Do()
    if((unsigned)recv_buf->Size()<1+proto_len+8+SHA1_DIGEST_SIZE) {
       if(recv_buf->Eof()) {
 	 if(recv_buf->Size()>0)
-	    LogError(1,"peer short handshake");
+	    LogError(1,_("peer short handshake"));
 	 else
-	    LogError(4,"peer closed connection");
+	    LogError(4,_("peer closed just accepted connection"));
 	 deleting=true;
 	 return MOVED;
       }
@@ -2584,7 +2662,7 @@ int TorrentJob::Do()
    if(!completed && torrent->Complete()) {
       if(parent->WaitsFor(this)) {
 	 PrintStatus(1,"");
-	 printf("Seeding in background...\n");
+	 printf(_("Seeding in background...\n"));
 	 parent->RemoveWaiting(this);
       }
       completed=true;
@@ -2599,45 +2677,55 @@ const char *TorrentTracker::Status() const
       return "";
    if(t_session->IsOpen())
       return t_session->CurrentStatus();
-   return xstring::format("next request in %s",NextRequestIn());
+   return xstring::format(_("next request in %s"),NextRequestIn());
 }
 
-void TorrentJob::PrintStatus(int v,const char *tab)
+xstring& TorrentJob::FormatStatus(xstring& s,int v,const char *tab)
 {
    const char *name=torrent->GetName();
    if(name)
-      printf("%sName: %s\n",tab,name);
-   printf("%s%s\n",tab,torrent->Status().get());
+      s.appendf("%sName: %s\n",tab,name);
+   s.appendf("%s%s\n",tab,torrent->Status().get());
    if(torrent->GetRatio()>0)
-      printf("%sratio: %f\n",tab,torrent->GetRatio());
+      s.appendf("%sratio: %f\n",tab,torrent->GetRatio());
+
    if(v>2) {
-      printf("%sinfo hash: %s\n",tab,torrent->GetInfoHash().dump());
-      printf("%stotal length: %llu\n",tab,torrent->TotalLength());
-      printf("%spiece length: %u\n",tab,torrent->PieceLength());
+      s.appendf("%sinfo hash: %s\n",tab,torrent->GetInfoHash().dump());
+      s.appendf("%stotal length: %llu\n",tab,torrent->TotalLength());
+      s.appendf("%spiece length: %u\n",tab,torrent->PieceLength());
    }
 
    if(v>1) {
       if(torrent->Trackers().count()==1) {
-	 printf("%stracker: %s - %s\n",tab,torrent->Trackers()[0]->GetURL(),
+	 s.appendf("%stracker: %s - %s\n",tab,torrent->Trackers()[0]->GetURL(),
 	       torrent->Trackers()[0]->Status());
       } else if(torrent->Trackers().count()>1) {
-	 printf("%strackers:\n",tab);
+	 s.appendf("%strackers:\n",tab);
 	 for(int i=0; i<torrent->Trackers().count(); i++) {
-	    printf("%s%2d. %s - %s\n",tab,i+1,torrent->Trackers()[i]->GetURL(),
+	    s.appendf("%s%2d. %s - %s\n",tab,i+1,torrent->Trackers()[i]->GetURL(),
 		  torrent->Trackers()[i]->Status());
 	 }
       }
    }
 
+   if(torrent->ShuttingDown())
+      return s;
+
    if(torrent->GetPeersCount()<=5 || v>1) {
       const TaskRefArray<TorrentPeer>& peers=torrent->GetPeers();
-      for(int i=0; i<peers.count(); i++)
-	 printf("%s  %s: %s\n",tab,peers[i]->GetName(),peers[i]->Status());
+      int not_connected_peers=torrent->GetPeersCount()-torrent->GetConnectedPeersCount();
+      if(v<=2 && not_connected_peers>0)
+	 s.appendf("%s  not connected peers: %d\n",tab,not_connected_peers);
+      for(int i=0; i<peers.count(); i++) {
+	 if(peers[i]->Connected() || v>2)
+	    s.appendf("%s  %s: %s\n",tab,peers[i]->GetName(),peers[i]->Status());
+      }
    } else {
-      printf("%s  peers:%d active:%d complete:%d\n",tab,
-	 torrent->GetPeersCount(),torrent->GetActivePeersCount(),
-	 torrent->GetCompletePeersCount());
+      s.appendf("%s  peers:%d connected:%d active:%d complete:%d\n",tab,
+	 torrent->GetPeersCount(),torrent->GetConnectedPeersCount(),
+	 torrent->GetActivePeersCount(),torrent->GetCompletePeersCount());
    }
+   return s;
 }
 
 void TorrentJob::ShowRunStatus(const SMTaskRef<StatusLine>& s)
@@ -2666,6 +2754,8 @@ CDECL_END
 
 CMD(torrent)
 {
+   Torrent::ClassInit();
+
 #define args (parent->args)
 #define eprintf parent->eprintf
    enum {
@@ -2702,7 +2792,7 @@ CMD(torrent)
    }
    args->back();
 
-   xstring_ca torrent_opt(args->Combine(0,args->getindex()));
+   xstring_ca torrent_opt(args->Combine(0,args->getindex()+1));
 
    xstring_ca cwd(xgetcwd());
    if(output_dir) {
@@ -2758,6 +2848,7 @@ CMD(torrent)
 #ifdef MODULE_CMD_TORRENT
 void module_init()
 {
+   Torrent::ClassInit();
    CmdExec::RegisterCommand("torrent",cmd_torrent);
 }
 #endif
