@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 2009-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: Torrent.h,v 1.29 2011/03/10 07:00:04 lav Exp $ */
+/* $Id: Torrent.h,v 1.32 2011/05/10 05:55:14 lav Exp $ */
 
 #ifndef TORRENT_H
 #define TORRENT_H
@@ -224,6 +223,7 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    RateLimit rate_limit;
    bool RateLow(RateLimit::dir_t dir) { return rate_limit.Relaxed(dir); }
 
+   int connected_peers_count;
    int active_peers_count;
    int complete_peers_count;
    int am_interested_peers_count;
@@ -262,6 +262,8 @@ class Torrent : public SMTask, protected ProtoLog, public ResClient
    TorrentPeer *FindPeerById(const xstring& p_id);
 
 public:
+   static void ClassInit();
+
    Torrent(const char *mf,const char *cwd,const char *output_dir);
 
    int Do();
@@ -290,6 +292,7 @@ public:
 
    const xstring& GetInfoHash() const { return info_hash; }
    int GetPeersCount() const { return peers.count(); }
+   int GetConnectedPeersCount() const { return connected_peers_count; }
    int GetActivePeersCount() const { return active_peers_count; }
    int GetCompletePeersCount() const { return complete_peers_count; }
 
@@ -627,7 +630,7 @@ public:
    ~TorrentJob();
    int Do();
    int Done() { return done; }
-   void PrintStatus(int v,const char *tab);
+   xstring& FormatStatus(xstring&,int v,const char *tab);
    void ShowRunStatus(const SMTaskRef<StatusLine>& s);
    int AcceptSig(int);
    void PrepareToDie();
