@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2011 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: commands.cc,v 1.306 2011/04/11 04:52:10 lav Exp $ */
+/* $Id: commands.cc,v 1.308 2011/06/16 12:24:05 lav Exp $ */
 
 #include <config.h>
 
@@ -471,7 +470,7 @@ const struct CmdExec::cmd_rec CmdExec::static_cmd_table[]=
    {"user",    cmd_user,   N_("user <user|URL> [<pass>]"),
 	 N_("Use specified info for remote login. If you specify URL, the password\n"
 	 "will be cached for future usage.\n")},
-   {"version", cmd_ver,    "version",
+   {"version", cmd_ver,    0,
 	 N_("Shows lftp version\n")},
    {"wait",    cmd_wait,   N_("wait [<jobno>]"),
 	 N_("Wait for specified job to terminate. If jobno is omitted, wait\n"
@@ -764,7 +763,9 @@ Job *CmdExec::builtin_lftp()
 	 cmd.set("version;");
 	 break;
       case('f'):
-	 cmd.vset("source \"",unquote(optarg),"\";",NULL);
+	 cmd.set("source ");
+	 cmd.append_quoted(optarg);
+	 cmd.append(';');
 	 break;
       case('c'):
 	 cmd.set_allocated(args->CombineCmd(args->getindex()-1));
@@ -904,31 +905,31 @@ Job *CmdExec::builtin_open()
 
    if(!no_bm && host && (bm=lftp_bookmarks.Lookup(host))!=0)
    {
-      char *cmd=string_alloca(5+3+3+xstrlen(user)*2+1+xstrlen(pass)*2+3+xstrlen(port)*2+strlen(bm)*2+6+1);
-      strcpy(cmd,"open -B ");
+      xstring& cmd=xstring::get_tmp("open -B ");
       if(user)
       {
-	 strcat(cmd,"-u \"");
-	 unquote(cmd+strlen(cmd),user);
+	 cmd.append("-u ");
+	 cmd.append_quoted(user);
 	 if(pass)
 	 {
-	    strcat(cmd,",");
-	    unquote(cmd+strlen(cmd),pass);
+	    cmd.append(",");
+	    cmd.append_quoted(pass);
 	 }
-	 strcat(cmd,"\" ");
+	 cmd.append(' ');
       }
       if(port)
       {
-	 strcat(cmd,"-p \"");
-	 unquote(cmd+strlen(cmd),port);
-	 strcat(cmd,"\" ");
+	 cmd.append("-p ");
+	 cmd.append_quoted(port);
+	 cmd.append(' ');
       }
 
-      strcat(cmd,bm);
+      cmd.append(bm);
 
       if(background)
-	 strcat(cmd," &\n");
-      strcat(cmd,";\n");
+	 cmd.append(" &\n");
+      else
+	 cmd.append(";\n");
 
       PrependCmd(cmd);
    }
@@ -1043,11 +1044,11 @@ Job *CmdExec::builtin_open()
       if(old)
       {
 	 bool is_file=false;
-	 const char *url=0;
+	 const char *old_url=0;
 	 if(url::is_url(old))
 	 {
 	    ParsedURL u(old,true);
-	    url=old;
+	    old_url=old;
 	    old=alloca_strdup(u.path);
 	    if(url::dir_needs_trailing_slash(u.proto))
 	       is_file=(last_char(old)!='/');
@@ -1057,17 +1058,15 @@ Job *CmdExec::builtin_open()
 	    if(url::dir_needs_trailing_slash(session->GetProto()))
 	       is_file=(last_char(old)!='/');
 	 }
-	 session->SetCwd(FileAccess::Path(old,is_file,url));
+	 session->SetCwd(FileAccess::Path(old,is_file,old_url));
       }
 
       const char *cd_arg=(url && url->orig_url)?url->orig_url.get():path;
-      char *s=string_alloca(strlen(cd_arg)*2+40);
-      strcpy(s,"&& cd \"");
-      unquote(s+strlen(s),cd_arg);
-      strcat(s,"\"");
+      xstring& s=xstring::get_tmp("&& cd ");
+      s.append_quoted(cd_arg);
       if(background)
-	 strcat(s,"&");
-      strcat(s,"\n");
+	 s.append('&');
+      s.append('\n');
       PrependCmd(s);
    }
 
@@ -1229,7 +1228,11 @@ Job *CmdExec::builtin_queue()
 	       queue->Suspend();
 	    }
 	    else
-	       queue->PrintStatus(2,"");
+	    {
+	       xstring& buf=xstring::get_tmp("");
+	       queue->FormatStatus(buf,2,"");
+	       printf("%s",buf.get());
+	    }
 	    exit_code=0;
 	    break;
 	 }
@@ -1268,13 +1271,11 @@ Job *CmdExec::builtin_queue()
 	 }
 
 	 if(!arg)
-	    queue->queue_feeder->DelJob(-1, verbose); /* delete the last job */
+	    exit_code=!queue->queue_feeder->DelJob(-1, verbose); /* delete the last job */
 	 else if(atoi(arg) != 0)
-	    queue->queue_feeder->DelJob(atoi(arg)-1, verbose);
+	    exit_code=!queue->queue_feeder->DelJob(atoi(arg)-1, verbose);
 	 else
-	    queue->queue_feeder->DelJob(arg, verbose);
-
-	 exit_code=0;
+	    exit_code=!queue->queue_feeder->DelJob(arg, verbose);
       }
       break;
 
@@ -1298,12 +1299,11 @@ Job *CmdExec::builtin_queue()
 	 }
 
 	 if(atoi(arg) != 0) {
-	    queue->queue_feeder->MoveJob(atoi(arg)-1, to, verbose);
+	    exit_code=!queue->queue_feeder->MoveJob(atoi(arg)-1, to, verbose);
 	    break;
 	 }
 
-	 queue->queue_feeder->MoveJob(arg, to, verbose);
-	 exit_code=0;
+	 exit_code=!queue->queue_feeder->MoveJob(arg, to, verbose);
       }
       break;
    }
@@ -1354,22 +1354,13 @@ CMD(ls)
    if(!nlist && args->count()==1 && var_ls[0])
       args->Append(var_ls);
 
-   bool use_color=false;
-   if(!nlist)
-   {
-      ResValue color=ResMgr::Query("color:use-color",0);
-      if(!strcasecmp(color,"auto"))
-	 use_color=!output && isatty(1);
-      else
-	 use_color=color.to_bool();
-   }
    bool no_status=(!output || output->usesfd(1));
 
    FileCopyPeer *src_peer=0;
    if(!nlist)
    {
       FileCopyPeerDirList *dir_list=new FileCopyPeerDirList(session->Clone(),args.borrow());
-      dir_list->UseColor(use_color);
+      dir_list->UseColor(ResMgr::QueryTriBool("color:use-color",0,(!output && isatty(1))));
       src_peer=dir_list;
    }
    else
@@ -1537,6 +1528,8 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
 
    time_fmt.set(0);
    if(time_style && time_style[0]) {
+      if (mode & DATE)
+	 need_exact_time=true;
       if(time_style[0]=='+')
 	 time_fmt.set(time_style+1);
       else if(!strcmp(time_style,"full-iso"))
@@ -1548,20 +1541,6 @@ const char *FileSetOutput::parse_argv(const Ref<ArgV>& a)
 	 time_fmt.set("%Y-%m-%d \n%m-%d %H:%M");
       else
 	 time_fmt.set(time_style);
-      need_exact_time=false;
-      if(time_fmt) {
-	 static const char exact_fmts[][3]={"%H","%M","%S","%N",""};
-	 int sep=strcspn(time_fmt,"\n|");
-	 for(int i=0; exact_fmts[i][0]; i++) {
-	    const char *f=strstr(time_fmt,exact_fmts[i]);
-	    if(!f)
-	       continue;
-	    if(i>1 || sep>f-time_fmt) {
-	       need_exact_time=true;
-	       break;
-	    }
-	 }
-      }
    }
 
    // remove parsed options.
@@ -1925,28 +1904,33 @@ CMD(jobs)
    }
    exit_code=0;
    args->back();
-   const char *arg=args->getnext();
-   if(!arg) {
-      parent->ListJobs(v);
-      return 0;
-   }
    const char *op=args->a0();
-   for(; arg; arg=args->getnext()) {
-      if(!isdigit((unsigned char)*arg)) {
-	 eprintf(_("%s: %s - not a number\n"),op,arg);
-	 exit_code=1;
-	 continue;
+   const char *arg=args->getnext();
+   xstring s("");
+   if(!arg) {
+      parent->FormatJobs(s,v);
+   } else {
+      for(; arg; arg=args->getnext()) {
+	 if(!isdigit((unsigned char)*arg)) {
+	    eprintf(_("%s: %s - not a number\n"),op,arg);
+	    exit_code=1;
+	    continue;
+	 }
+	 int n=atoi(arg);
+	 Job *j=parent->FindJob(n);
+	 if(!j) {
+	    eprintf(_("%s: %d - no such job\n"),op,n);
+	    exit_code=1;
+	    continue;
+	 }
+	 j->FormatOneJob(s,v);
       }
-      int n=atoi(arg);
-      Job *j=parent->FindJob(n);
-      if(!j) {
-	 eprintf(_("%s: %d - no such job\n"),op,n);
-	 exit_code=1;
-	 continue;
-      }
-      j->ListOneJob(v);
    }
-   return 0;
+   if(exit_code)
+      return 0;
+   OutputJob *out=new OutputJob(output.borrow(), args->a0());
+   Job *j=new echoJob(s,s.length(),out);
+   return j;
 }
 
 CMD(cd)
@@ -2591,7 +2575,7 @@ CMD(help)
 CMD(ver)
 {
    printf(
-      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2011);
+      _("LFTP | Version %s | Copyright (c) 1996-%d Alexander V. Lukyanov\n"),VERSION,2012);
    printf("\n");
    printf(
       _("LFTP is free software: you can redistribute it and/or modify\n"
@@ -3257,10 +3241,7 @@ CMD(get1)
 	 const char *bn=basename_ptr(src);
 	 if(bn[0]!='/')
 	 {
-	    char *dst1=string_alloca(strlen(dst)+strlen(bn)+1);
-	    strcpy(dst1,dst);
-	    strcat(dst1,bn);
-	    dst=dst1;
+	    dst=xstring::get_tmp(dst).append(bn);
 	    auto_rename=true;
 	 }
       }
@@ -3282,11 +3263,7 @@ CMD(get1)
 	       slash++;
 	    else
 	       slash=src;
-	    char *dst1=string_alloca(strlen(dst)+strlen(slash)+2);
-	    strcpy(dst1,dst);
-	    strcat(dst1,"/");
-	    strcat(dst1,slash);
-	    dst=dst1;
+	    dst=xstring::cat(dst,"/",slash,NULL);
 	 }
       }
    }
