@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,11 +14,10 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
-/* $Id: complete.cc,v 1.82 2011/03/25 14:35:43 lav Exp $ */
+/* $Id: complete.cc,v 1.84 2011/06/16 11:09:32 lav Exp $ */
 
 #include <config.h>
 
@@ -356,11 +355,10 @@ static completion_type cmd_completion_type(const char *cmd,int start)
       if(alias && !TouchedAlias::IsTouched(alias,used_aliases))
       {
 	 used_aliases=new TouchedAlias(alias,used_aliases);
-	 char *cmd1=string_alloca(strlen(alias)+strlen(w)-strlen(buf)+1);
-	 strcpy(cmd1,alias);
-	 strcat(cmd1,w+strlen(buf));
-	 start=start-strlen(buf)+strlen(alias);
-	 cmd=cmd1;
+	 int buf_len=strlen(buf);
+	 const char *cmd1=xstring::cat(alias,w+buf_len,NULL);
+	 cmd=alloca_strdup(cmd1);
+	 start=start-buf_len+strlen(alias);
 	 continue;
       }
       const char *full=CmdExec::GetFullCommandName(buf);
@@ -396,6 +394,7 @@ static completion_type cmd_completion_type(const char *cmd,int start)
    || !strcmp(buf,"ls")
    || !strcmp(buf,"cls")
    || !strcmp(buf,"du")
+   || !strcmp(buf,"edit")
    || !strcmp(buf,"find")
    || !strcmp(buf,"recls")
    || !strcmp(buf,"rels")
@@ -666,7 +665,7 @@ static char **lftp_completion (const char *text,int start,int end)
       bool tilde_expanded=false;
       const char *home=getenv("HOME");
       int home_len=xstrlen(home);
-      pat=(char*)alloca((len+home_len)*2+10);
+      pat=string_alloca((len+home_len)*2+10);
       if(len>0 && home_len>0 && text[0]=='~' && (len==1 || text[1]=='/'))
       {
 	 glob_quote(pat,home,home_len);
@@ -742,7 +741,7 @@ static char **lftp_completion (const char *text,int start,int end)
 	 break;
       }
 
-      pat=(char*)alloca(len*2+10);
+      pat=string_alloca(len*2+10);
       glob_quote(pat,text,len);
 
       if(pat[0]=='~' && pat[1]==0)
@@ -1237,13 +1236,12 @@ static int lftp_complete_remote(int count,int key)
 
 int lftp_rl_getc(FILE *file)
 {
-   int res;
+   SignalHook::DoCount(SIGINT);
    CharReader r(fileno(file));
-
    for(;;)
    {
       SMTask::Schedule();
-      res=r.GetChar();
+      int res=r.GetChar();
       if(res==r.EOFCHAR)
 	 return EOF;
       if(res!=r.NOCHAR)
