@@ -1,7 +1,7 @@
 /*
- * lftp and utils
+ * lftp - file transfer program
  *
- * Copyright (c) 1996-2010 by Alexander V. Lukyanov (lav@yars.free.net)
+ * Copyright (c) 1996-2012 by Alexander V. Lukyanov (lav@yars.free.net)
  *
  * This program is free software; you can redistribute it and/or modify
  * it under the terms of the GNU General Public License as published by
@@ -14,8 +14,7 @@
  * GNU General Public License for more details.
  *
  * You should have received a copy of the GNU General Public License
- * along with this program; if not, write to the Free Software
- * Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.
+ * along with this program.  If not, see <http://www.gnu.org/licenses/>.
  */
 
 /* $Id: misc.cc,v 1.102 2011/03/25 14:35:43 lav Exp $ */
@@ -805,10 +804,10 @@ const char *memrchr(const char *buf,char c,size_t len)
    return 0;
 }
 
-const char *shell_encode(const char *string)
+const xstring& shell_encode(const char *string)
 {
    if(!string)
-      return 0;
+      return xstring::null;
 
    static xstring result;
 
